<?php

namespace Drupal\elementpay\Events;

use Drupal\elementpay\Provider\Provider;
use Symfony\Component\HttpFoundation\JsonResponse;

/** Class Base Event for all EPS events */
abstract class BaseEvent {
  protected $payment;
  protected $provider;
  protected $webhook_signing_key;

  public function __construct($payment, $webhook_signing_key) {
    $this->provider = new Provider();
    $this->payment = $payment;
    $this->webhook_signing_key = $webhook_signing_key;
  }

  public abstract function handle($event);

  protected function hashResponse($response) {
    return hash_hmac('sha1', $response, $this->webhook_signing_key);
  }

    /**
     * @param $code
     * @param $message
     * @return JsonResponse
     */
  protected function sendResponse($code, $message) : JsonResponse {
    $response = [
      'response' => [
        'status' => $code,
        'message' => $message,
        'timestamp' => time()
      ]
    ];
    $response['hash'] = $this->hashResponse(json_encode($response['response']));

    return new JsonResponse($response, $code);
  }

    /**
     * @param $event
     * @return bool
     */
  protected function isOrderValid($event) : bool {
    $order = $this->payment->getOrder();
    if (!($isInvalid = isset($order))) {
      \Drupal::logger('elementpay')->alert('EPS Webhooks: the order#' . $event['id'] . 'doesn\'t exist');
    }
    return $isInvalid;
  }

  protected function isOrderCanceled() : bool {
    $order = $this->payment->getOrder();
    $state = $order->getState()->getValue();
    if ($isCanceled = ($state['value'] === 'canceled')) {
      \Drupal::logger('elementpay')->alert('EPS Webhooks: the order#' . $order->id() . ' has already been canceled');
    }
    return $isCanceled;
  }

  protected function isOrderPaid() : bool {
    $state = $this->payment->getOrder()->getState()->getValue();
    return $state['value'] === 'completed';
  }

  protected function isHashValid($event) : bool {
    if (!isset($event['hash'])) {
      \Drupal::logger('elementpay')->alert('EPS Webhooks: invalid hash');
      return false;
    }

    $hash = $event['hash'];
    unset($event['hash']);
    if ($hash !== $this->hashResponse(http_build_query($event, null, '&', PHP_QUERY_RFC3986))) {
      \Drupal::logger('elementpay')->alert('EPS Webhooks: invalid hash');
      return false;
    }
    return true;
  }

  protected function amountMatching($event) : bool {
    $order = $this->payment->getOrder();
    $amount = sprintf("%.2f", $order->getTotalPrice()->getNumber());
    if ($amountNotMatches = ($amount !== $event['amount'])) {
      \Drupal::logger('elementpay')->alert('EPS Webhooks: The amount of order ' . $order->id() . " doesn't match.");
    }
    return !$amountNotMatches;
  }
}
