<?php

namespace Drupal\elementpay\Provider;

/**
 * Curl wrapper for Yii
 */
class Curl {
  /**
   * Curl config to overlap a default conf
   *
   * @var array
   */
  public $options = [];

  /**
   * Default conf
   *
   * @var array
   */
  protected $_config = [
    CURLOPT_RETURNTRANSFER => 1,
    CURLOPT_AUTOREFERER    => 1,
    CURLOPT_FORBID_REUSE   => 1,
    CURLOPT_FRESH_CONNECT  => 1,
    CURLOPT_HEADER         => 1,
  ];

  /**
   * Default headers
   *
   * @var array
   */
  public $headers = [
    'Accept-Language' => 'en-us,en;q=0.5',
    'Accept-Charset'  => 'utf-8;q=0.7,*;q=0.7',
    'Accept'          => 'application/json',
    'Cache-Control'   => 'max-age=0',
    'Pragma'          => '',
    'Keep-Alive'      => '300',
    'Connection'      => 'keep-alive',
  ];

  /**
   * Cookies collector
   *
   * @var array
   */
  protected $cookie = [];

  /**
   * Response data collector
   *
   * @var array
   */
  protected $response = [];

  /**
   * Request timeout
   *
   * @var int
   */
  public $timeout = 60;

  /**
   * Use proxy
   *
   * @var bool
   */
  public $proxy = false;

  /**
   * @inheritdoc
   */
  public function __construct()
  {
    $this->mergeOptions($this->_config);
  }

  /**
   * Merges options
   *
   * @param array $target
   */
  protected function mergeOptions(&$target)
  {
    if ($this->options) {
      foreach ($this->options as $key => $value) {
        $target[$key] = $value;
      }
      $this->options = [];
    }
  }

  /**
   * Sends request and return results wit response headers and data
   *
   * @param string $url
   *
   * @return array
   */
  protected function exec($url)
  {
    $options = $this->_config;
    $this->mergeOptions($options);

    $options[CURLOPT_URL] = $url;
    $options[CURLOPT_REFERER] = $url;
    if (!isset($options[CURLOPT_HTTPHEADER])) {
      $headers = [];
      foreach ($this->headers as $name => $value) {
        $headers[] = $name . ':' . $value;
      }
      $options[CURLOPT_HTTPHEADER] = $headers;
      unset($headers, $name, $header);
    }

    if (!empty($this->cookie)) {
      $cookie = implode(';', $this->cookie);
      $options[CURLOPT_COOKIE] = $cookie;
    } else {
      $options[CURLOPT_FOLLOWLOCATION] = 1;
    }

    if (!empty($this->proxy)) {
      $options[CURLOPT_PROXY] = $this->proxy;
      $options[CURLOPT_HTTPPROXYTUNNEL] = 1;
    }

    $options[CURLOPT_CONNECTTIMEOUT] = $this->timeout;
    $options[CURLOPT_USERAGENT] = 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.22 (KHTML, like Gecko) Chrome/25.0.1364.97 Safari/537.22' . date('His');

    $ch = curl_init();
    curl_setopt_array($ch, $options);

    $response = curl_exec($ch);
    $error = curl_error($ch);

    $respData = [
      'http_code'  => curl_getinfo($ch, CURLINFO_HTTP_CODE),
      'header'     => '',
      'body'       => '',
      'curl_error' => '',
      'last_url'   => '',
    ];

    if (!empty($error)) {
      $respData['curl_error'] = $error;
    } else {
      $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);

      $header = substr($response, 0, $header_size);
      $respData['header'] = $this->parseResponseHeader($header);

      if (!isset($options[CURLOPT_NOBODY]) && !isset($options[CURLOPT_FILE])) {
        $respData['body'] = substr($response, $header_size);
        $content_type = curl_getinfo($ch, CURLINFO_CONTENT_TYPE);
        if (stripos($content_type, 'json') !== false) {
          $respData['raw_body'] = $respData['body'];
          $respData['body'] = json_decode($respData['body'], true);
        }
      }

      $respData['last_url'] = curl_getinfo($ch, CURLINFO_EFFECTIVE_URL);

      $this->cookie = preg_match_all('#Set-Cookie: (?P<cookie>\S+)#', $header, $matches) ? $matches['cookie'] : '';
    }
    curl_close($ch);
    unset($ch, $options, $header, $response, $error);

    return $respData;
  }

  /**
   * Parses response headers to array
   *
   * @param string $header_text
   *
   * @return array
   */
  protected function parseResponseHeader($header_text)
  {
    $headers = [];
    $parsed = explode("\r\n", $header_text);
    foreach ($parsed as $i => $line) {
      if ($i === 0) {
        $headers['http_code'] = $line;
      } elseif ($line) {
        $rows = explode(': ', $line);
        $headers[$rows[0]] = sizeof($rows) == 2 ? $rows[1] : '';
      }
    }

    return $headers;
  }

  /**
   * Sends HEAD request
   *
   * @param string $url
   * @param array $params
   *
   * @return array
   * @throws \Exception
   */
  public function head($url, $params = [])
  {
    $this->setOption(CURLOPT_NOBODY, true);

    return $this->exec($this->buildUrl($url, $params));
  }

  /**
   * Sends request by any HTTP method
   *
   * @param string $url
   * @param string $method
   * @param array  $params
   *
   * @return array
   */
  public function request($url, $method, $params)
  {
    $method = strtolower($method);

    return is_callable([$this, $method]) ?
      $this->{$method}($url, $params) : [];
  }

  /**
   * Sends GET request
   *
   * @param string $url
   * @param array $params
   *
   * @return array
   * @throws \Exception
   */
  public function get($url, $params = [])
  {
    $this->setOption(CURLOPT_HTTPGET, true);

    return $this->exec($this->buildUrl($url, $params));
  }

  /**
   * Sends POST request
   *
   * @param string $url
   * @param array  $data
   *
   * @return array
   */
  public function post($url, $data = [])
  {
    $this->setOption(CURLOPT_POST, true);
    $this->setOption(CURLOPT_POSTFIELDS, http_build_query($data));

    return $this->exec($url);
  }

  /**
   * Sends POST file request
   *
   * @param string $url
   * @param array  $data
   *
   * @return array
   */
  public function postFile($url, $data = [])
  {
    $this->setOption(CURLOPT_POST, true);
    $this->setOption(CURLOPT_POSTFIELDS, $data);

    return $this->exec($url);
  }

  /**
   * Sends PUT request
   *
   * @param string $url
   * @param array $data
   * @param array $params
   *
   * @return array
   * @throws \Exception
   */
  public function put($url, $data, $params = [])
  {
    // write to memory/temp
    $f = fopen('php://temp', 'rw+');
    fwrite($f, $data);
    rewind($f);

    $this->setOption(CURLOPT_PUT, true);
    $this->setOption(CURLOPT_INFILE, $f);
    $this->setOption(CURLOPT_INFILESIZE, strlen($data));

    return $this->exec($this->buildUrl($url, $params));
  }

  /**
   * Builds url for request
   *
   * @param string $url
   * @param array $data
   *
   * @return string
   * @throws \Exception
   */
  public function buildUrl($url, $data = [])
  {
    if (!($parsed = parse_url($url))) {
      throw new \Exception('Wrong url format');
    }

    if ($data) {
      if (!empty($parsed['query'])) {
        parse_str($parsed['query'], $parsed['query']);
      } else {
        $parsed['query'] = [];
      }

      $params = array_merge($parsed['query'], $data);
      $parsed['query'] = ($params) ? '?' . http_build_query($params) : '';
      unset($params);
    }

    if (!isset($parsed['path'])) {
      $parsed['path'] = '/';
    }

    $port = '';
    if (isset($parsed['port'])) {
      $port = ':' . $parsed['port'];
    }

    return $parsed['scheme'] . '://' . $parsed['host'] . $port . $parsed['path'] . (isset($parsed['query']) ? $parsed['query'] : '');
  }

  /**
   * Set request header
   *
   * @param string $name
   * @param string $header
   */
  public function setHeader($name, $header)
  {
    $this->headers[$name] = $header;
  }

  /**
   * Drop request header
   *
   * @param string $name
   */
  public function dropHeader($name)
  {
    if (isset($this->headers[$name])) {
      unset($this->headers[$name]);
    }
  }


  /**
   * Set request headers from array
   *
   * @param array $headers
   */
  public function setHeaders($headers)
  {
    $this->headers = $headers;
  }

  /**
   * Set request option
   *
   * @param string     $option
   * @param string|int $value
   *
   * @return $this
   */
  public function setOption($option, $value)
  {
    $this->options[$option] = $value;

    return $this;
  }

  /**
   * Set cookie for request
   *
   * @param string      $name
   * @param string|null $value
   */
  public function setCookie($name, $value = null)
  {
    if (is_array($name)) {
      foreach ($name as $k => $v) {
        $this->cookie[] = rawurlencode($k).'='.rawurlencode($v);
      }
    } else {
      $cookie = rawurlencode($name) . '=' . rawurlencode($value);
      if (!empty($this->cookie)) {
        foreach ($this->cookie as $k => $v) {
          $key = substr($v, 0, strpos($v, '='));
          if ($key == $name) {
            $this->cookie[$k] = $cookie;
            break;
          }
        }
      } else {
        $this->cookie[] = $cookie;
      }
    }
  }

  /**
   * Retrieves cookies from last response
   *
   * @return array
   */
  public function getCookie()
  {
    return $this->cookie;
  }
}

