<?php

namespace Drupal\elementpay\Provider;

/** EPS class which provides all communication with EPS API! */
class Provider {


  private $url;
  private $public_key;
  private $private_key;

  public function __construct() {
    $this->public_key = PS_PUBLIC_KEY;
    $this->private_key = PS_PRIVATE_KEY;
    $this->url = PS_MODE === 'test' ? PS_SANDBOX_LINK : PS_LIVE_LINK;
  }

  private function hashData($method, $params) : string {
    $string_for_hash = $method . http_build_query($params, null, '&', PHP_QUERY_RFC3986);
    return hash_hmac('sha1', $string_for_hash, $this->private_key);
  }

  public function hashResponse($response) : string {
    return hash_hmac('sha1', $response, $this->private_key);
  }

  public function getMethodList() : ?array {
    if ($cache = \Drupal::cache()->get('elementpay_method_list')) {
      return $cache->data;
    }

    $getMethodsURL = $this->url . 'getMethods';
    $params = [
      'key' => $this->public_key,
      'timestamp' => time(),
    ];
    $params['hash'] = $this->hashData('getMethods?', $params);

    $ch = new Curl();
    $result = $ch->post($getMethodsURL, $params);

    if (
      isset($result['error']) ||
      !empty($result['curl_error']) ||
      isset($result['body']['error']) ||
      $result['http_code'] === 502
    ) {
      \Drupal::logger('elementpay')->alert('EPS Provider: Error response of getMethodList');
      return null;
    }
    \Drupal::cache()->set('elementpay_method_list', $result['body']['response']['methods'], \Drupal::time()->getRequestTime() + (86400));
    return $result['body']['response']['methods'];
  }

  public function getServicesCategories() : ?array {
    if ($cache = \Drupal::cache()->get('elementpay_categories')) {
      return $cache->data;
    }

    $getServicesURL = $this->url . "getCategories";

    $params = [
      'key' => $this->public_key,
      'timestamp' => time(),
    ];
    $params['hash'] = $this->hashData('getCategories?', $params);

    $ch = new Curl();
    $result = $ch->post($getServicesURL, $params);

    if (
      isset($result['error']) ||
      !empty($result['curl_error']) ||
      isset($result['body']['error']) ||
      $result['http_code'] === 502
    ) {
      \Drupal::logger('elementpay')->alert(
        'EPS Provider: Error response of getServicesCategories. ' . var_dump($this->private_key)
      );
      return null;
    }
    $categories = [];
    foreach ($result['body']['response']['categories'] as $category) {
      $categories[$category['key']] = $category;
    }

    \Drupal::cache()->set('elementpay_method_list', $categories, \Drupal::time()->getRequestTime() + (86400));
    return $categories;
  }

  public function initPayment($service_id, $amount, $order, $url_success, $url_waiting, $url_reject, $attrs = []) : ?array {
    $initPaymentURL = $this->url . 'initPayment?';
    $ch = new Curl();

    $params = [
      'service_id' => $service_id,
      'amount' => $amount,
      'order' => $order,
      '_successUrl' => $url_success,
      '_waitingUrl' => $url_waiting,
      '_rejectUrl' => $url_reject,
      'key' => $this->public_key,
      'timestamp' => time()
    ];

    foreach ($attrs as $key => $value) {
      $params[$key] = $value;
    }

    $params['hash'] = $this->hashData('initPayment?', $params);

    $result = $ch->post($initPaymentURL, $params);
    if (isset($result['error']) || !empty($result['curl_error'])) {
      \Drupal::logger('elementpay')->alert('EPS Provider: Error response of initPayment');
      return null;
    }

    if (array_key_exists('error', $result['body'])) {
      \Drupal::logger('elementpay')->alert(
        'EPS Provider: Error response of initPayment (' . $result['body']['error']['message'] . ').'
      );
      return $result['body'];
    }

    return $result['body']['response'];
  }

  public function getPaymentStatus($payment_id) : ?int {
    $getStatusURL = $this->url . 'getStatus?';
    $params = [
      'payment_id' => $payment_id,
      'key' => $this->public_key,
      'timestamp' => time(),
    ];
    $params['hash'] = $this->hashData('getStatus?', $params);

    $ch = new Curl();
    $result = $ch->post($getStatusURL, $params);

    if (isset($result['error']) || !empty($result['curl_error'])) {
      \Drupal::logger('elementpay')->alert('EPS Provider: Error response of getPaymentStatus');
      return null;
    }

    if (isset($result['body']['error'])) {
      \Drupal::logger('elementpay')->alert(
        'EPS Provider: Error response of getPaymentStatus (' . $result['body']['error']['message'] . ').'
      );
      return null;
    }
    return $result['body']['response']['status'];
  }
}
